var readFile, writeFile, watchFile, stat, Bundler, Task, TaskPool, _ref;
_ref = require('fs'), readFile = _ref.readFile, writeFile = _ref.writeFile, watchFile = _ref.watchFile, stat = _ref.stat;
Bundler = require('./bundler').Bundler;
function writeResult(path, data, cbk){
  return writeFile(path, data, 'utf-8', function(err){
    if (err) {
      console.log(path + ": " + err);
      return;
    }
    return typeof cbk == 'function' ? cbk() : void 8;
  });
}
/**
 *  @class Task
 */
Task = (function(){
  /**
   *  @constructor
   */
  Task.displayName = 'Task';
  var prototype = Task.prototype, constructor = Task;
  function Task(_arg){
    var _ref;
    this.dependencies = (_ref = _arg.dependencies) != null
      ? _ref
      : [], this.callback = _arg.callback, this.watch = _arg.watch;
    this.watching = false;
  }
  /**
   *
   */
  prototype.run = (function(){
    function run(){
      var f, _i, _ref, _len, _this = this;
      if (this.watch && !this.watching) {
        this.watching = true;
        for (_i = 0, _len = (_ref = this.dependencies).length; _i < _len; ++_i) {
          f = _ref[_i];
          watchFile(f, _fn);
        }
      }
      return this.callback();
      function _fn(curr, prev){
        if (curr.mtime > prev.mtime) {
          return _this.callback();
        }
      }
    }
    return run;
  }());
  return Task;
}());
/**
 *  @class TaskPool
 */
TaskPool = (function(){
  /**
   *  @constructor
   */
  TaskPool.displayName = 'TaskPool';
  var prototype = TaskPool.prototype, constructor = TaskPool;
  function TaskPool(_arg){
    this.watch = _arg.watch;
    this.tasks = [];
  }
  /**
   *
   */
  prototype.createTaskForFile = (function(){
    function createTaskForFile(file, outfile, callback){
      function fileCallback(error){
        return stat(file, function(err_in, istat){
          return stat(outfile, function(err_out, ostat){
            if (!err_out && istat.mtime <= ostat.mtime) {
              console.log("= " + file);
              return;
            }
            return readFile(file, 'utf-8', function(err, data){
              var res;
              if (err) {
                console.log(file + ": " + err);
                return;
              }
              try {
                res = callback(data);
                return writeResult(outfile, res, function(){
                  return console.log(new Date().toLocaleTimeString() + ": " + file + " => " + outfile);
                });
              } catch (e) {
                return console.log(file + ": " + e);
              }
            });
          });
        });
      }
      return this.addTask(new Task({
        dependencies: [file],
        callback: fileCallback
      }));
    }
    return createTaskForFile;
  }());
  /**
   *
   */
  prototype.createBundleTask = (function(){
    function createBundleTask(outfile, files, extensions){
      var bundler;
      bundler = new Bundler({
        files: files,
        watch: this.watch,
        extensions: extensions
      });
      return this.addTask(new Task({
        dependencies: [],
        callback: function(){
          return bundler.bundle(function(err, data){
            return writeResult(outfile, data, function(){
              return console.log("* Wrote " + outfile);
            });
          });
        }
      }));
    }
    return createBundleTask;
  }());
  /**
   *
   */
  prototype.addTask = (function(){
    function addTask(task){
      this.tasks.push(task);
      task.watch = this.watch;
      task.run();
      return task;
    }
    return addTask;
  }());
  return TaskPool;
}());
exports.Task = Task;
exports.TaskPool = TaskPool;